class Utils():
    def __init__(self):
        self.inputs = {}
        self.outputs = {}
        self.hiddens = {}
        self.relations = [[],[]]
        self.units = [self.inputs,self.outputs]
        self.default = [0,-0.2]

    def dtanh(self,y):
         return 1.0-y*y

    def getstrength(self,input,output,layer):
        key = "%s:%s" % (input,output)
        unit = self.units[layer]
        default = self.default[layer]
        return unit[key] if key in unit else default

    def setstrength(self,input,output,layer,strength):
        key = "%s:%s" % (input,output)
        unit = self.units[layer]
        unit[key] = strength
        self.relations[layer].append(input)

    def generatesynapses(self,inputs,outputs):
        if len(inputs)>3: return None
        index = len(self.hiddens)+1
        key = ':'.join([str(input) for input in inputs])
        if key not in self.hiddens:
            self.hiddens[key] = index
            for input in inputs: self.setstrength(input,index,0,1.0/len(inputs))
            for output in outputs: self.setstrength(index,output,1,0.1)

    def getallsynapses(self,inputs,outputs):
        l={}
        for input in inputs:
            for value in [key.split(':')[1] for key in self.inputs.keys() if str(input) in key]: l[value]=1
        for output in outputs:
            for value in [key.split(':')[1] for key in self.inputs.keys() if str(output) in key]: l[value]=1
        return l.keys()

    def setupnetwork(self,inputs,outputs):
        # unit lists
        self.iu=inputs
        self.hu=self.getallsynapses(inputs,outputs)
        self.ou=outputs
        # synapse values
        self.si = [1.0]*len(self.iu)
        self.sh = [1.0]*len(self.hu)
        self.so = [1.0]*len(self.ou)
        # weights matrix
        self.i = [[self.getstrength(input,hidden,0) for hidden in self.hu] for input in self.iu]
        self.o = [[self.getstrength(hidden,output,1) for output in self.ou] for hidden in self.hu]

    def updatedataset(self):
        for i in range(len(self.iu)):
            for j in range(len(self.hu)):
                self.setstrength(self.iu[i],self.hu[j],0,self.i[i][j])
        for j in range(len(self.hu)):
            for k in range(len(self.ou)):
                self.setstrength(self.hu[j],self.ou[k],1,self.o[j][k])

